unit UMain;
//Requires the MQTT Client Library. Read here: jamiei.com/blog/code/mqtt-client-library-for-delphi/

{$MODE DELPHI}
// DO NOT USE: {$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls,
  // Own Units
  UMisc,
  // MQTT
  MQTT,
  // JSON
  fpjson, jsonparser,
  Clipbrd,
  DateUtils;

const
  CAPPTITLE = 'B4R Experiment ESP8266 DHT11 MQTT v20161207';
  CAPPCOPYRIGHT = 'by rwbLinn.de';
  // Define the MQTT topics to subscribe to
  MQTTTopicTemperature = 'dht11/temperature';
  MQTTTopicHumidity = 'dht11/humidity';

type

  { TfrmMain }

  TfrmMain = class(TForm)
    btnClearLog: TButton;
    btnConnect: TButton;
    btnDisconnect: TButton;
    gbMQTTControl: TGroupBox;
    gbMQTTLog: TGroupBox;
    lbledHumidity: TLabeledEdit;
    lbMQTTLog: TListBox;
    lblConnected: TLabel;
    lblCopyright: TLabel;
    lbledMQTTHost: TLabeledEdit;
    lbledMQTTPort: TLabeledEdit;
    lbledTemperature: TLabeledEdit;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure btnDisconnectClick(Sender: TObject);
    procedure btnClearLogClick(Sender: TObject);
    procedure OnPublish(Sender: TObject; topic, payload: string);
  private
    { private declarations }
  public
    { public declarations }
    MQTTClient: TMQTTClient;
    IsConnected: Boolean;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.lfm}

{ TfrmMain }

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  frmMain.Caption := CAPPTITLE;
  lblCopyright.Caption := CAPPCOPYRIGHT;
  lbledMQTTHost.Text := '192.168.0.10';
  lbledMQTTPort.Text := '1883';
  // btnConnectClick(Sender);
end;

procedure TfrmMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  btnDisconnectClick(Sender);
  //DO NOT USE BELOW as giving runtime error
  //MQTTClient.Free
  CloseAction := caFree;
end;

// Connect to the MQTT Host (Broker) and subscribe to topics
procedure TfrmMain.btnConnectClick(Sender: TObject);
begin
  MQTTClient := TMQTTClient.Create(lbledMQTTHost.Text, StrToInt(lbledMQTTPort.Text));
  MQTTClient.OnPublish := OnPublish;
  MQTTClient.Connect;
  lblConnected.Caption := 'Connected';
  // Subscribtions
  MQTTClient.Subscribe(MQTTTopicTemperature);
  MQTTClient.Subscribe(MQTTTopicHumidity);
  IsConnected := True;
end;

procedure TfrmMain.btnDisconnectClick(Sender: TObject);
begin
  if IsConnected = False Then Exit;
    Try
        MQTTClient.DisConnect;
        lblConnected.Caption := 'Not Connected';
    Except
      ShowMessage('Can not disconnect from MQTT.');
    end;
end;

procedure TfrmMain.btnClearLogClick(Sender: TObject);
begin
  lbMQTTLog.Items.Clear;
end;

//Received a message. show topic and the payload
procedure TfrmMain.OnPublish(Sender: TObject; topic, payload: string);
begin
  lbMQTTLog.Items.Add(TimeNow + ': '+ topic + ' = ' + payload);
  If Topic = MQTTTopicTemperature Then lbledTemperature.Text := payload;
  If Topic = MQTTTopicHumidity Then lbledHumidity.Text := payload;
end;

end.

